% This file is released under the terms of the GNU General Public License,
% version 3. See http://www.gnu.org/licenses/gpl.html

function [varianceEnvelope, meanEnvelope] = calculateEnvelopeVariance(data, hpass, lpass, finalPassBandwidth)
    persistent ham signalInfo
    if isempty(ham)
        signalInfo = nbt_Info;
        signalInfo.converted_sample_frequency = 1000;
        signalInfo.original_sample_frequency = 1000;
        ham = hamming(2^13);
    end

    if nargin == 5
        uppass = hpass;
        lowpass = lpass;
    else
        [a,b] = pwelch(data,ham,[],[],1000);
        startind = find(b>hpass,1,'first');
        endind = find(b>lpass,1,'first');
        [~, maxind] = max(a(startind:endind));
        maxind = maxind + startind - 1;
        uppass = (-finalPassBandwidth + sqrt(finalPassBandwidth^2 + 4*b(maxind)^2))/2;
        lowpass = uppass + finalPassBandwidth;
    end

    [AmplitudeEnvelope, ~] = nbt_GetAmplitudeEnvelope(data, signalInfo, uppass, lowpass, 2/uppass);
    
    varianceEnvelope = var(AmplitudeEnvelope);
    meanEnvelope = mean(AmplitudeEnvelope);
end
