% This file is released under the terms of the GNU General Public License,
% version 3. See http://www.gnu.org/licenses/gpl.html

function [  ] = plot_avalanche_sim_vs_theoretical_kappa( path,timeSeries )
    data=timeSeries';

    [aval_size, aval_dur] = avalanches(data,0,median(data)/2);
    noBins = 20; 
    
    minX = 1; maxX = 10000;
    avs = aval_size(aval_size >= minX & aval_size <= maxX);
    ta = 10.^(linspace(log10(minX),log10(maxX),noBins));
    for i = 1:noBins
        fb(i) = (1 - (minX/maxX)^0.5)^-1 * (1 - (minX/ta(i))^0.5);
    end
    
    for i = 1:noBins
        ys(i) = nnz(avs<ta(i));
    end
    
    for i = 1:noBins
        if i==1
            ysk(i) = nnz(avs<ta(i));
        else
            ysk(i) = nnz(avs<ta(i) & avs>ta(i-1));
        end
    end
    
    for i = 1:noBins
        if i==1
            fbk(i)=fb(i);
        else
            fbk(i)=fb(i)-fb(i-1);
        end
    end
    
    fig=figure('name','Avalanche kappa');
    set(fig,'PaperPositionMode','auto');
    set(fig,'Position',[100, 100, 1000, 800]);
    set(fig,'Visible','off');
%    set(fig,'Visible','off');

    ys = ys / length(avs);
    ysk = ysk / length(avs);
    kappa = 1+ sum(fb - ys)/noBins;
    subplot(2,2,1);
    loglog(ta,ys, ta, fb)
    box off
    title(['Kappa: ' num2str(kappa)])
    xlabel('Avalanche Size (Neurons)')
    ylabel('CDF')
    leg = legend({'Simulation', 'Theoretical'});
    set(leg,'box','off');

    minX = 1; maxX = 100;
    avs = aval_dur(aval_dur >= minX & aval_dur <= maxX);
    td = 10.^(linspace(log10(minX),log10(maxX),noBins));
    for i = 1:noBins
        fb(i) = (1 - (minX/maxX)^1)^-1 * (1 - (minX/td(i))^1);
    end
    
    for i = 1:noBins
        ys(i) = nnz(avs<td(i));
    end
    
    for i = 1:noBins
        if i==1
            yskd(i) = nnz(avs<td(i));
        else
            yskd(i) = nnz(avs<td(i) & avs>td(i-1));
        end
    end
    
    for i = 1:noBins
        if i==1
            fbkd(i)=fb(i);
        else
            fbkd(i)=fb(i)-fb(i-1);
        end
    end
    
    yskd = yskd/length(avs);
    ys = ys / length(avs);
    kappa = 1+ sum(fb - ys)/noBins;
    subplot(2,2,2);
    loglog(td,ys, td, fb)
    box off
    title(['Kappa: ' num2str(kappa)])
    xlabel('Avalanche Duration (ms)')
    ylabel('CDF')
    leg = legend({'Simulation', 'Theoretical'});
    set(leg,'box','off');
    
    subplot(2,2,3);
    loglog(ta,ysk, ta, fbk)
    box off
    %title(['Kappa: ' num2str(kappa)])
    xlabel('Avalanche Size (Neurons)')
    ylabel('CDF')
    leg = legend({'Simulation', 'Theoretical'});
    set(leg,'box','off');
    
    subplot(2,2,4);
    loglog(td,yskd, td, fbkd)
    box off
    %title(['Kappa: ' num2str(kappa)])
    xlabel('Avalanche Duration (s)')
    ylabel('CDF')
    leg = legend({'Simulation', 'Theoretical'});
    set(leg,'box','off');
    
    print(strcat(path,'Avalanche_statistics_kappa.png'),'-dpng');
end