% This file is released under the terms of the GNU General Public License,
% version 3. See http://www.gnu.org/licenses/gpl.html

function [filestruct] = run_analysis( my_path )
    try
        if my_path(length(my_path))~='/' && my_path(length(my_path))~='\'
            my_path = [my_path filesep];
        end
        
        [s,mess,messid] = mkdir(my_path,'plots');
        plots_path = [my_path 'plots' filesep];
        mkdir(strcat(my_path,'data/'));

 		parameters_dict=read_dictionary_from_file(strcat(my_path,'Simulation.Parameters'));

 		dfa_calc_smallest_window = parameters_dict('DFA calc smallest window');
 		dfa_calc_largest_window = parameters_dict('DFA calc largest window');
 		dfa_calc_frequency_low = parameters_dict('DFA calc frequency low');
 		dfa_calc_frequency_high = parameters_dict('DFA calc frequency high');
 		dfa_calc_window_overlap = parameters_dict('DFA calc window overlap');
 		dfa_fit_smallest_window = parameters_dict('DFA fit smallest window');
 		dfa_fit_largest_window = parameters_dict('DFA fit largest window');
 		kappa_avalanche_size_min = parameters_dict('Kappa avalanche size min');
 		kappa_avalanche_size_max = parameters_dict('Kappa avalanche size max');
 		kappa_bins = parameters_dict('Kappa bins');
        kappa_threshold_type = parameters_dict('Kappa threshold type');
 		plot_dfa=parameters_dict('Plot DFA');
 		plot_kappa=parameters_dict('Plot Kappa');
        calc_DFA=parameters_dict('Calculate DFA');
        calc_kappa=parameters_dict('Calculate Kappa');
        calc_envelope_variance=parameters_dict('Calculate Envelope Variance');
        calc_all_power_info=parameters_dict('Calculate all power info');

        add_whitenoise_first=parameters_dict('Add white noise');
        white_noise_sd=parameters_dict('White noise sd');

		external_input=parameters_dict('External input');
        stimulus_size=parameters_dict('Stimulus size');
        
        PSD_fit=parameters_dict('PSD fit');
        PSD_fit_plot=parameters_dict('PSD fit plot');
        PSD_plot=parameters_dict('PSD plot');
        PSD_fit_interval1_low=parameters_dict('PSD fit interval1 low');
        PSD_fit_interval1_high=parameters_dict('PSD fit interval1 high');
        PSD_fit_interval2_low=parameters_dict('PSD fit interval2 low');
        PSD_fit_interval2_high=parameters_dict('PSD fit interval2 high');
        PSD_fit_hamming_exponent=parameters_dict('PSD fit hamming exponent');
        
        max_avalanche_size_ksstat=parameters_dict('Max avalanche size KS-stat');

        %TODO: get data sampling rate from settings
        sampling_rate = 1000;

		%shuffle random number generator
		rng('shuffle');

		time_series_txt_file_path = strcat(my_path,'Time.Series');

        simulation_data_file_path = strcat(my_path,'SimulationData.h5');

        if exist(time_series_txt_file_path, 'file') == 2
            timeSeries = importdata(time_series_txt_file_path,' ');
        else
            timeSeries = h5read(simulation_data_file_path,'/data/population_spiking_main_network');
            if size(timeSeries,1)<size(timeSeries,2)
                timeSeries = timeSeries';
            end
            disp(size(timeSeries));
        end
		
        
        if isempty(timeSeries)
            return;
        end
        
        timeSeriesLength = size(timeSeries,1);
        
        if size(timeSeries,2)==2
            excInhTimeSeries = timeSeries;
            timeSeries = timeSeries(1:timeSeriesLength,1)+timeSeries(1:timeSeriesLength,2);
            
            plot_excinh_firing(plots_path,excInhTimeSeries);
        end
        
        if add_whitenoise_first
            timeSeries = add_white_noise(timeSeries,white_noise_sd);
        end
        
        mean_act = mean(timeSeries);

		if external_input
            try               
                stimulus_timeSeries = h5read(simulation_data_file_path,'/data/population_spiking_external_input_network');
                appendCorrectPLFNoise(my_path,timeSeries,stimulus_timeSeries);
            catch E
                disp(getReport(E));
            end
        end
        
        if calc_DFA
            try
            	%traditional DFA calculation
                [x,y]=calculateDfaPoints(timeSeries,sampling_rate,[dfa_calc_smallest_window,dfa_calc_largest_window],dfa_calc_window_overlap,dfa_calc_frequency_low,dfa_calc_frequency_high);
                DFA_exp=fitDFA(x',y,[dfa_fit_smallest_window,dfa_fit_largest_window]);

                DFA_full_exp_matrix = horzcat(x,y);
                dlmwrite(strcat(my_path,'data/DFA_full_exponents.txt'),DFA_full_exp_matrix);

                if plot_dfa
                    dfaBendPlot(plots_path,x,y,[dfa_fit_smallest_window,dfa_fit_largest_window]);
                end

            catch E
                DFA_exp=NaN;
                disp(getReport(E));
            end
        end
		
        if calc_kappa
            try
                if kappa_threshold_type==1
                    threshold=median(timeSeries)/2;
                else
                    threshold=0;
                end

                [KappaSize,KappaDur,num_avalanches] = calculateKappa(timeSeries',threshold,kappa_bins,kappa_avalanche_size_min,kappa_avalanche_size_max);


                %TODO: get rid of redundancy here
                [aval_size, aval_dur] = avalanches(timeSeries',0,threshold);
                [~, silences_dur] = silences(timeSeries',0,threshold);
                [kstat_opt,powerlaw_shift] = kstat_optimize(aval_size,24,max_avalanche_size_ksstat);
                [kstat_opt_dur,powerlaw_shift_dur] = kstat_optimize(aval_dur,24,max_avalanche_size_ksstat);
                
                disp(['kstat_opt',num2str(powerlaw_shift)]);
                
                powerlaw_shift_silences=0;
                try
                    [kstat_opt_silences,powerlaw_shift_silences] = kstat_optimize(silences_dur,24,max_avalanche_size_ksstat);
                catch
                end
                
                if plot_kappa
                    plot_avalanche_sim_vs_theoretical(plots_path,aval_size,max_avalanche_size_ksstat,powerlaw_shift,'_size');
                    plot_avalanche_sim_vs_theoretical(plots_path,aval_dur,max_avalanche_size_ksstat,powerlaw_shift_dur,'_duration');
                    try
                        plot_avalanche_sim_vs_theoretical(plots_path,silences_dur,max_avalanche_size_ksstat,powerlaw_shift_silences,'_silences_duration');
                    catch
                    end
                end

                
            catch E
                KappaSize=NaN;
                KappaDur=NaN;
                kstat_opt=NaN;
                disp(getReport(E));
            end
        end
        
        if calc_all_power_info
            try
                [names,values]=get_all_power_info(timeSeries);

            catch E
                disp(getReport(E));
            end
        end
        

        calc_amplitude = true;

        if calc_amplitude

            nfft=2^10; %number of fast fourier transforms, higher this number and the frequency resolution of the spectrum goes up,
            number_of_channels=1;

            Signal = timeSeries;

            %% determine intervals at which power is calculated

            FS=1000;

            interval_Hz(1,:)=[1 4];
            interval_Hz(2,:)=[4 8];
            interval_Hz(3,:)=[8 12];
            interval_Hz(4,:)=[12 30];
            interval_Hz(5,:)=[30 45];

            [p,f]=pwelch(randn(1,nfft),hamming(nfft),0,nfft,FS);

            for i=1:5
                interval{i}=find(f>interval_Hz(i,1)&f<interval_Hz(i,2));
            end

            nr_interv=length(interval);

            %% calculated integrated and normalized power for all channels

            integrated=zeros(nr_interv,number_of_channels);
            normalized=zeros(nr_interv,number_of_channels);

            for i=1:number_of_channels
                [p,f]=pwelch(Signal(:,i),hamming(nfft),0,nfft,FS);
                p=sqrt(p); % transform power to amplitude
                
                %%%%  plot spectrum for each channel, push enter to see all
                
                %             intervalall=[];
                %             for i=1:5
                %                 intervalall=[intervalall;interval{i}];
                %             end
                %             plot(f(intervalall),p(intervalall))
                %             drawnow
                %             input('')
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                for j=1:nr_interv
                    integrated(j,i) = mean(p(interval{j}));
                    normalized(j,i) = mean(p(interval{j}))/sum([mean(p([interval{1}]));mean(p([interval{2}]));mean(p([interval{3}]));mean(p([interval{4}]));mean(p([interval{5}]))]);
                end
            end
        end

        
        if calc_envelope_variance
            try
                [varianceEnvelope, meanEnvelope] = calculateEnvelopeVariance( timeSeries, 0.5, 8, 16 );

                %TODO: store this somewhere
            catch E
                disp(getReport(E));
            end
        end
        
        PSD_fit = false;
        if PSD_fit
            [exp1,exp2,exp_full]=Scaling_PSD(plots_path,timeSeries,sampling_rate,PSD_fit_hamming_exponent,PSD_fit_interval1_low,PSD_fit_interval1_high,PSD_fit_interval2_low,PSD_fit_interval2_high,PSD_fit_plot);
        end
     
		fid = fopen(strcat(my_path,'CriticalityFitness.txt'), 'w');
		
        if calc_DFA
            fprintf(fid, 'DFA=%f\n', DFA_exp);
        end

        if calc_amplitude
            fprintf(fid, '1_4_absolute=%f\n4_8_absolute=%f\n8_12_absolute=%f\n12_30_absolute=%f\n30_45_absolute=%f\n',integrated(1,1),integrated(2,1),integrated(3,1),integrated(4,1),integrated(5,1));
            fprintf(fid, '1_4_normalized=%f\n4_8_normalized=%f\n8_12_normalized=%f\n12_30_normalized=%f\n30_45_normalized=%f\n',normalized(1,1),normalized(2,1),normalized(3,1),normalized(4,1),normalized(5,1));
        end

        if calc_kappa
            fprintf(fid, 'KappaSize=%f\nKappaDur=%f\nKStat=%f\n', KappaSize, KappaDur, kstat_opt);
        end
        
        if PSD_fit
            fprintf(fid, 'exp1=%f\nexp2=%f\nexp_full=%f\n', exp1,exp2,exp_full);
        end
        
        fprintf(fid, 'mean_act=%f\n', mean_act);
        
        if calc_kappa
            fprintf(fid, 'KStatDur=%f\n', kstat_opt_dur);
        end
   		
   		if calc_envelope_variance
   			fprintf(fid, 'varianceEnvelope=%f\nmeanEnvelope=%f\n', varianceEnvelope, meanEnvelope);
   		end
		fclose(fid);

    catch E
        disp(getReport(E));
    end
end
