#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from brian2 import *
from collections import Counter
from scipy.signal import welch

def get_separated_excinh_spike_times(s_mon,types,numcycles):
    return get_separated_excinh_spike_times_interval(s_mon,types,0,numcycles)

def get_separated_excinh_spike_times_interval(s_mon,types,startcycle,endcycle,start_index=0):
    spike_ms_times = s_mon.it[1]/ms

    numcycles = endcycle - startcycle
    spikes_per_cycle_exc = np.zeros(numcycles)

    start_index=0

    found_types = np.where (  np.logical_and ( types[np.array(s_mon.it[0])] == 1, np.logical_and(s_mon.t[start_index:]/msecond>startcycle , s_mon.t[start_index:]/msecond < endcycle ))  )
    #found_types = np.where( types[np.array(s_mon.it[0])] == 1)
    if len(found_types)>0:
        selected_spikes = spike_ms_times[found_types]
        for i in xrange(len(selected_spikes)):
            spikes_per_cycle_exc[ int(selected_spikes[i]-startcycle) ]+=1

    spikes_per_cycle_inh = np.zeros(numcycles)
    found_types = np.where (  np.logical_and ( types[np.array(s_mon.it[0])] == 0, np.logical_and(s_mon.t[start_index:]/msecond>startcycle , s_mon.t[start_index:]/msecond < endcycle ))  )
    #found_types = np.where( types[np.array(s_mon.it[0])] == 0)
    if len(found_types)>0:
        selected_spikes = spike_ms_times[found_types]
        for i in xrange(len(selected_spikes)):
            spikes_per_cycle_inh[ int(selected_spikes[i]-startcycle) ]+=1

    return [spikes_per_cycle_exc,spikes_per_cycle_inh]

def get_neuron_spikes_in_interval(s_mon,num_neurons,startcycle,endcycle,start_index=0):
    # t0 = time.time()
    spikes_per_neuron = np.zeros(num_neurons)

    found_values = np.where (  np.logical_and(s_mon.t[start_index:]/msecond>=startcycle , s_mon.t[start_index:]/msecond <= endcycle )  )[0].astype(int)+start_index

    if(len(found_values)==0):
        last_index=0+start_index
    else:
        last_index = len(found_values)+start_index
        found_neurons = s_mon.it[0][found_values]
        counts = Counter(found_neurons.tolist())
        spikes_per_neuron[counts.keys()]=counts.values()

    # t2 = time.time()
    # print t1-t0,t2-t1
    return {'spikes_per_neuron':spikes_per_neuron,'next_index':last_index}


def get_spike_times_interval(s_mon,startcycle,endcycle):
    spike_ms_times = s_mon.it[1]/ms

    numcycles = endcycle - startcycle
    spikes_per_cycle = np.zeros(numcycles)

    start_index=0

    for i in xrange(len(spike_ms_times)):
        spikes_per_cycle[spike_ms_times[i]]+=1

    return spikes_per_cycle

def get_weights_averaged_per_step(weights_monitor):
    return np.mean(weights_monitor.w, axis=0)

def get_weights_change_times(weights_monitor):
    weight_changes_times = []

    #i is neuron
    for i in xrange(np.shape(weights_monitor.w)[0]):
        last_weight_value = weights_monitor.w[i,0]
        last_weight_time = 0
        #j is time
        for j in xrange(1,np.shape(weights_monitor.w)[1]):
            crt_weight_value =weights_monitor.w[i,j]
            if(crt_weight_value!=last_weight_value):
                last_weight_value = crt_weight_value
                weight_changes_times.append(j-last_weight_time)
                last_weight_time = j

    return weight_changes_times

#on x axis are units
#on y axis is time
def get_histogram_evolution(evolution_vector,min_val,max_val,num_bins=50):
    indexes_p_small = np.where(evolution_vector<min_val)
    evolution_vector[indexes_p_small]=min_val

    indexes_p_small = np.where(evolution_vector>=max_val)
    evolution_vector[indexes_p_small]=max_val

    dims = np.shape(evolution_vector)

    histogram = np.zeros((num_bins,dims[1]),dtype=np.float)

    for i in xrange(dims[1]):
        [hist,bin_edges]=np.histogram(evolution_vector[:,i],range=(min_val,max_val),bins=num_bins)
        histogram[:,i]= hist

    histogram = log10(histogram)
    histogram = np.flipud(histogram)

    return histogram

def get_welch(time_series,resolution=2048):
    f, Pxx_den = welch(time_series,nperseg=resolution,fs=1000)
    return [f, Pxx_den]