#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

#TODO: refactor

import pyqtgraph as pg
from PyQt4 import QtGui, QtCore
import threading
import scipy.ndimage
from brian2 import *
import cros_model.params as p
from cros_model.data_analysis.data_processing import get_separated_excinh_spike_times_interval, \
    get_neuron_spikes_in_interval, get_welch
import math

plot_width = 400
color_rgb_exc = [235,52,52]
color_rgb_inh = [0,25,149]

power_spectrum_color=[0,0,0]

# color_rgb_exc = [255,0,0]
# color_rgb_inh = [0,0,255]


class GridLabel(QtGui.QLabel):
    def __init__(self,plot_data):
        QtGui.QLabel.__init__(self)
        self.plot_data = plot_data

    def mousePressEvent(self,event):
        self.plot_data.set_central_neuron(int(round(event.x()/ (plot_width/50.))),int(round(event.y()/(plot_width/50.))))

def run_app(plot_data):
    print "got in the thread"
    app = QtGui.QApplication([])
    win = pg.GraphicsWindow(title="Basic plotting examples")

    plot_configuration = plot_data.plot_configuration

    # 2 default plots (grid spikes and grid activity value, and activity evolution)
    plot_count = 3
    if plot_configuration[p.SHOW_POWER_PLOT]:
        plot_count += 1
    if plot_configuration[p.SHOW_STATE_MONITOR]:
        plot_count += 1
    if plot_configuration[p.SHOW_HISTOGRAM_STATE]:
        plot_count += 1
    if plot_configuration[p.SHOW_U_EXC_EXC_SUM_PLOT]:
        plot_count += 1
    if plot_configuration[p.SHOW_X_EXC_EXC_SUM_PLOT]:
        plot_count += 1

    number_columns = int(math.ceil(float(plot_count) / 2))

    win.resize(plot_width * number_columns,plot_width * 2)

    win.setWindowTitle('CROS model UI')

    layout = QtGui.QGridLayout()
    win.setLayout(layout)

    # Enable antialiasing for prettier plots
    pg.setConfigOptions(antialias=True)
    pg.setConfigOption('background', 'w')
    pg.setConfigOption('foreground', 'k')

    spike_grid_ui = GridLabel(plot_data)
    spike_grid_ui.setFixedSize(plot_width, plot_width)
    spike_grid_ui.setAutoFillBackground(True)
    spike_grid_ui.setScaledContents(True)
    spike_grid_ui.setMouseTracking(True)
    spike_grid_ui.setFixedWidth(plot_width)

    btn_minus = QtGui.QPushButton('-')
    speed_label = QtGui.QLabel(plot_configuration[p.PLOT_SPEED])
    speed_label.setAutoFillBackground(True)
    btn_plus = QtGui.QPushButton('+')

    activity_grid_ui = GridLabel(plot_data)
    activity_grid_ui.setFixedSize(plot_width, plot_width)
    activity_grid_ui.setAutoFillBackground(True)
    activity_grid_ui.setScaledContents(True)
    activity_grid_ui.setFixedWidth(plot_width)

    spikes_evolution_ui = pg.PlotWidget(title="Spikes evolution")
    curve_exc = spikes_evolution_ui.plot(pen=color_rgb_exc)
    curve_inh = spikes_evolution_ui.plot(pen=color_rgb_inh)
    spikes_evolution_ui.setFixedWidth(plot_width)
    spikes_evolution_ui.setFixedHeight(plot_width)

    layout.addWidget(spike_grid_ui, 0, 0, 1, 3)
    layout.addWidget(activity_grid_ui, 1, 0, 1, 3)
    layout.addWidget(btn_minus, 2, 0, 1, 1)
    layout.addWidget(speed_label, 2, 1, 1, 1)
    layout.addWidget(btn_plus, 2, 2, 1, 1)
    layout.addWidget(spikes_evolution_ui, 0, 3, 1, 3)

    # 0 based index of plot number. plots are drawn in order of indexes, filling each column in order left to right, top bottom
    crt_plot_idx = 2

    if plot_configuration[p.SHOW_HISTOGRAM_STATE]:
        weights_histogram_ui = pg.PlotWidget(title="Neuron activation histogram")
        weights_histogram_ui.setFixedWidth(plot_width)
        weights_histogram_ui.setFixedHeight(plot_width)

        ## make interesting distribution of values
        vals = np.hstack([np.random.normal(size=500), np.random.normal(size=260, loc=4)])

        ## compute standard histogram
        y, x = np.histogram(vals, bins=np.linspace(-3, 8, 40))

        ## notice that len(x) == len(y)+1
        ## We are required to use stepMode=True so that PlotCurveItem will interpret this data correctly.

        max_histogram_bin = int(len(plot_data.types)/5)

        curve_hist = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=(100, 100, 100, 255))
        weights_histogram_ui.setYRange(0,max_histogram_bin)
        weights_histogram_ui.addItem(curve_hist)
        curve_hist.setData(x, y)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(weights_histogram_ui, widget_pos_row, widget_pos_col, 1, 3)

    if plot_configuration[p.SHOW_POWER_PLOT]:
        # matplotlib widget is quite slow. try to avoid it, or run it in parallel thread.
        # matplotwidget = MatplotlibWidget()
        # power_subplot = matplotwidget.getFigure().add_subplot(111)
        # matplotwidget.draw()
        powerplot_ui = pg.PlotWidget(title="Power spectrum (x-frequency, y-log10(power)")
        powerplot_curve = powerplot_ui.plot(pen=power_spectrum_color)
        powerplot_ui.setFixedWidth(plot_width)
        powerplot_ui.setFixedHeight(plot_width)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(powerplot_ui, widget_pos_row, widget_pos_col, 1, 3)

    if plot_configuration[p.SHOW_LOG_LOG_POWER_PLOT]:
        # matplotlib widget is quite slow. try to avoid it, or run it in parallel thread.
        # matplotwidget = MatplotlibWidget()
        # power_subplot = matplotwidget.getFigure().add_subplot(111)
        # matplotwidget.draw()
        powerplot_log_log_ui = pg.PlotWidget(title="Power spectrum (x-log10(frequency), y-log10(power)")
        powerplot_log_log_1f_line = powerplot_log_log_ui.plot(pen=pg.mkPen('r',width=2,style=QtCore.Qt.DotLine))
        powerplot_log_log_curve = powerplot_log_log_ui.plot(pen=power_spectrum_color)
        powerplot_log_log_1f_line.setData([0,3],[3,0])
        powerplot_log_log_ui.setFixedWidth(plot_width)
        powerplot_log_log_ui.setFixedHeight(plot_width)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(powerplot_log_log_ui, widget_pos_row, widget_pos_col, 1, 3)

    if plot_configuration[p.SHOW_STATE_MONITOR]:
        state_monitor_evolution_ui = pg.PlotWidget(title="State monitor")
        state_monitor_evolution_ui.setFixedWidth(plot_width)
        state_monitor_evolution_ui.setFixedHeight(plot_width)
        neuron_monitor_curves_array = []
        for i in xrange(50):
            if plot_data.get_data()['types'][i] == 1:
                curve2 = state_monitor_evolution_ui.plot(pen='b')
            else:
                curve2 = state_monitor_evolution_ui.plot(pen='r')
            neuron_monitor_curves_array.append(curve2)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(state_monitor_evolution_ui, widget_pos_row, widget_pos_col, 1, 3)

    if plot_configuration[p.SHOW_U_EXC_EXC_SUM_PLOT]:        
        u_exc_exc_ui = GridLabel(plot_data)
        u_exc_exc_ui.setFixedSize(plot_width, plot_width)
        u_exc_exc_ui.setAutoFillBackground(True)
        u_exc_exc_ui.setScaledContents(True)
        u_exc_exc_ui.setFixedWidth(plot_width)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(u_exc_exc_ui, widget_pos_row, widget_pos_col, 1, 3)
        
    if plot_configuration[p.SHOW_X_EXC_EXC_SUM_PLOT]:        
        x_exc_exc_ui = GridLabel(plot_data)
        x_exc_exc_ui.setFixedSize(plot_width, plot_width)
        x_exc_exc_ui.setAutoFillBackground(True)
        x_exc_exc_ui.setScaledContents(True)
        x_exc_exc_ui.setFixedWidth(plot_width)
        crt_plot_idx += 1
        plot_col_idx = crt_plot_idx / 2
        plot_row_idx = crt_plot_idx % 2
        widget_pos_row = plot_row_idx
        widget_pos_col = plot_col_idx * 3
        layout.addWidget(x_exc_exc_ui, widget_pos_row, widget_pos_col, 1, 3)
    
    speeds = ["min","slow","medium","high","realtime"]

    def decrease_speed():
        crt_speed_idx = speeds.index(plot_data.get_speed())
        new_speed_idx = max(0, crt_speed_idx - 1)
        new_speed_text = speeds[new_speed_idx]
        speed_label.setText(new_speed_text)
        plot_data.set_speed(new_speed_text)

    def increase_speed():
        crt_speed_idx = speeds.index(plot_data.get_speed())
        new_speed_idx = min(len(speeds) - 1, crt_speed_idx + 1)
        new_speed_text = speeds[new_speed_idx]
        speed_label.setText(new_speed_text)
        plot_data.set_speed(new_speed_text)

    btn_minus.clicked.connect(decrease_speed)
    btn_plus.clicked.connect(increase_speed)

    def update():
        crt_data = plot_data.get_data()
        clocktime = crt_data['clocktime']

        types = plot_data.get_types()

        grid_size = int(math.sqrt(len(crt_data['types'])))

        # import pdb
        # pdb.set_trace()

        image_magnification_factor = 2

        image = np.zeros((grid_size, grid_size, 3)).astype(np.uint32) + 255

        image_resized = np.zeros(
            (grid_size * image_magnification_factor, grid_size * image_magnification_factor, 3)).astype(np.uint32) + 255

        act_exc = np.zeros((grid_size, grid_size))
        act_inh = np.zeros((grid_size, grid_size))

        indexes_exc = np.where(np.reshape(types, (grid_size, grid_size)) == 1)
        indexes_inh = np.where(np.reshape(types, (grid_size, grid_size)) == 0)

        spikes_integration_window = plot_configuration[p.SPIKES_INTEGRATION_WINDOW_MS]

        if np.shape(crt_data['grid_data'])[0] > 0:
            act_exc[indexes_exc[0], indexes_exc[1]] = ((crt_data['grid_data'][indexes_exc[0], indexes_exc[
                1]] + spikes_integration_window) * 255 / spikes_integration_window).astype(int)
            act_inh[indexes_inh[0], indexes_inh[1]] = ((crt_data['grid_data'][indexes_inh[0], indexes_inh[
                1]] + spikes_integration_window) * 255 / spikes_integration_window).astype(int)

            exc_act_idx = np.where(act_exc > 0)
            inh_act_idx = np.where(act_inh > 0)

            image[exc_act_idx[0], exc_act_idx[1], 0] = 255 - ( 255 - color_rgb_exc[0] )/255. * act_exc[exc_act_idx[0], exc_act_idx[1]]
            image[exc_act_idx[0], exc_act_idx[1], 1] = 255 - ( 255 - color_rgb_exc[1] )/255. * act_exc[exc_act_idx[0], exc_act_idx[1]]
            image[exc_act_idx[0], exc_act_idx[1], 2] = 255 - ( 255 - color_rgb_exc[2] )/255. * act_exc[exc_act_idx[0], exc_act_idx[1]]

            image[inh_act_idx[0], inh_act_idx[1], 0] = 255 - ( 255 - color_rgb_inh[0] )/255. *  act_inh[inh_act_idx[0], inh_act_idx[1]]
            image[inh_act_idx[0], inh_act_idx[1], 1] = 255 - ( 255 - color_rgb_inh[1] )/255. *  act_inh[inh_act_idx[0], inh_act_idx[1]]
            image[inh_act_idx[0], inh_act_idx[1], 2] = 255 - ( 255 - color_rgb_inh[2] )/255. *  act_inh[inh_act_idx[0], inh_act_idx[1]]

            image[:, :, 0] = image[:, :, 0].transpose()
            image[:, :, 1] = image[:, :, 1].transpose()
            image[:, :, 2] = image[:, :, 2].transpose()

            image_resized[:, :, 0] = scipy.ndimage.zoom(image[:, :, 0], image_magnification_factor, order=0)
            image_resized[:, :, 1] = scipy.ndimage.zoom(image[:, :, 1], image_magnification_factor, order=0)
            image_resized[:, :, 2] = scipy.ndimage.zoom(image[:, :, 2], image_magnification_factor, order=0)

            b = (
            255 << 24 | image_resized[:, :, 0] << 16 | image_resized[:, :, 1] << 8 | image_resized[:, :, 2]).flatten()

            QI = QtGui.QImage(b, grid_size * image_magnification_factor, grid_size * image_magnification_factor,
                              QtGui.QImage.Format_RGB32)

            spike_grid_ui.setPixmap(QtGui.QPixmap.fromImage(QI))
            spike_grid_ui.repaint()

        if np.shape(crt_data['grid_data2'])[0] > 0:
            #TODO: rescale interval for crt_data into image - and do that separately for the two types of model.

            if "Coba" in crt_data['simulation_configuration'][p.SIMULATION_TYPE] or "AExp" in crt_data['simulation_configuration'][p.SIMULATION_TYPE]:
                max_value = -50.0
                min_value = -80.0
            else:
                max_value = 0.1
                min_value = -0.5

            border_max_reset_indexes = np.where(crt_data['grid_data2']>max_value)
            border_min_reset_indexes = np.where(crt_data['grid_data2']<min_value)
            activation_grid_copy = crt_data['grid_data2'].copy()
            activation_grid_copy[border_max_reset_indexes]=max_value
            activation_grid_copy[border_min_reset_indexes]=min_value

            rescaled_grid_data = (activation_grid_copy-min_value)*255.0/((max_value-min_value))

            image[:,:, 0] = rescaled_grid_data
            image[:,:, 1] = rescaled_grid_data
            image[:,:, 2] = rescaled_grid_data

            image[:, :, 0] = image[:, :, 0].transpose()
            image[:, :, 1] = image[:, :, 1].transpose()
            image[:, :, 2] = image[:, :, 2].transpose()

            image_resized[:, :, 0] = scipy.ndimage.zoom(image[:, :, 0], image_magnification_factor, order=0)
            image_resized[:, :, 1] = scipy.ndimage.zoom(image[:, :, 1], image_magnification_factor, order=0)
            image_resized[:, :, 2] = scipy.ndimage.zoom(image[:, :, 2], image_magnification_factor, order=0)

            b = (
            255 << 24 | image_resized[:, :, 0] << 16 | image_resized[:, :, 1] << 8 | image_resized[:, :, 2]).flatten()

            QI = QtGui.QImage(b, grid_size * image_magnification_factor, grid_size * image_magnification_factor,
                              QtGui.QImage.Format_RGB32)

            activity_grid_ui.setPixmap(QtGui.QPixmap.fromImage(QI))
            activity_grid_ui.repaint()

            #activity_grid_ui.setImage(crt_data['grid_data2'])

        ###### U EXC EXC

        if plot_configuration[p.SHOW_U_EXC_EXC_SUM_PLOT] and plot_data.get_u_exc_exc_matrix()!=None:
            #TODO: rescale interval for crt_data into image - and do that separately for the two types of model.
            
            data = plot_data.get_u_exc_exc_matrix()
            max_value = 150
            min_value = 0

            border_max_reset_indexes = np.where(data>max_value)
            border_min_reset_indexes = np.where(data<min_value)
            activation_grid_copy = data.copy()
            activation_grid_copy[border_max_reset_indexes]=max_value
            activation_grid_copy[border_min_reset_indexes]=min_value

            rescaled_grid_data = (activation_grid_copy-min_value)*255.0/((max_value-min_value))

            image[:,:, 0] = rescaled_grid_data
            image[:,:, 1] = rescaled_grid_data
            image[:,:, 2] = rescaled_grid_data

            image[:, :, 0] = image[:, :, 0].transpose()
            image[:, :, 1] = image[:, :, 1].transpose()
            image[:, :, 2] = image[:, :, 2].transpose()

            image_resized[:, :, 0] = scipy.ndimage.zoom(image[:, :, 0], image_magnification_factor, order=0)
            image_resized[:, :, 1] = scipy.ndimage.zoom(image[:, :, 1], image_magnification_factor, order=0)
            image_resized[:, :, 2] = scipy.ndimage.zoom(image[:, :, 2], image_magnification_factor, order=0)

            b = (
            255 << 24 | image_resized[:, :, 0] << 16 | image_resized[:, :, 1] << 8 | image_resized[:, :, 2]).flatten()

            QI = QtGui.QImage(b, grid_size * image_magnification_factor, grid_size * image_magnification_factor,
                              QtGui.QImage.Format_RGB32)

            u_exc_exc_ui.setPixmap(QtGui.QPixmap.fromImage(QI))
            u_exc_exc_ui.repaint()

        if plot_configuration[p.SHOW_X_EXC_EXC_SUM_PLOT] and plot_data.get_x_exc_exc_matrix()!=None :
            #TODO: rescale interval for crt_data into image - and do that separately for the two types of model.
            
            data = plot_data.get_x_exc_exc_matrix()
            max_value = 150
            min_value = 0

            border_max_reset_indexes = np.where(data>max_value)
            border_min_reset_indexes = np.where(data<min_value)
            activation_grid_copy = data.copy()
            activation_grid_copy[border_max_reset_indexes]=max_value
            activation_grid_copy[border_min_reset_indexes]=min_value

            rescaled_grid_data = (activation_grid_copy-min_value)*255.0/((max_value-min_value))

            image[:,:, 0] = rescaled_grid_data
            image[:,:, 1] = rescaled_grid_data
            image[:,:, 2] = rescaled_grid_data

            image[:, :, 0] = image[:, :, 0].transpose()
            image[:, :, 1] = image[:, :, 1].transpose()
            image[:, :, 2] = image[:, :, 2].transpose()

            image_resized[:, :, 0] = scipy.ndimage.zoom(image[:, :, 0], image_magnification_factor, order=0)
            image_resized[:, :, 1] = scipy.ndimage.zoom(image[:, :, 1], image_magnification_factor, order=0)
            image_resized[:, :, 2] = scipy.ndimage.zoom(image[:, :, 2], image_magnification_factor, order=0)

            b = (
            255 << 24 | image_resized[:, :, 0] << 16 | image_resized[:, :, 1] << 8 | image_resized[:, :, 2]).flatten()

            QI = QtGui.QImage(b, grid_size * image_magnification_factor, grid_size * image_magnification_factor,
                              QtGui.QImage.Format_RGB32)

            x_exc_exc_ui.setPixmap(QtGui.QPixmap.fromImage(QI))
            x_exc_exc_ui.repaint()

        if len(crt_data['exc_spikes_data']) > 0:
            if clocktime - plot_configuration[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS]<0:
                min_time=0
                max_time = plot_configuration[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS]
            else:
                min_time=clocktime - plot_configuration[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS]
                max_time=clocktime

            curve_exc.setData(
                np.arange(min_time,max_time),
                crt_data['exc_spikes_data'])
            curve_inh.setData(
                np.arange(min_time,max_time),
                crt_data['inh_spikes_data'])

        if plot_configuration[p.SHOW_HISTOGRAM_STATE] and np.shape(crt_data['grid_data2'])[0] > 0:
            if "Coba" in crt_data['simulation_configuration'][p.SIMULATION_TYPE] or "AExp" in crt_data['simulation_configuration'][p.SIMULATION_TYPE]:
                max_value = -50
                min_value = -80
            else:
                max_value = 0.5
                min_value = -0.5

            grid_data_array = np.reshape(crt_data['grid_data2'], (grid_size * grid_size))
            grid_data_finite = grid_data_array[np.isfinite(grid_data_array)]

            y, x = np.histogram(grid_data_finite,bins=np.linspace(min_value,max_value,50))
            curve_hist.setData(x, y)

        if crt_data['frequency_values'] is not None and np.max(crt_data['frequency_power'])>0:
            if plot_configuration[p.SHOW_POWER_PLOT]:
                powerplot_curve.setData(
                crt_data['frequency_values'], log10(crt_data['frequency_power']))
            if plot_configuration[p.SHOW_LOG_LOG_POWER_PLOT]:
                log10f = log10(crt_data['frequency_values'][1:])
                power_log10f = log10(crt_data['frequency_power'][1:])
                interval_of_interest = np.where( np.logical_and( log10f>=0 , log10f<=2) )

                selected_log10f = log10f[interval_of_interest]
                selected_pxx_denlog = power_log10f[interval_of_interest]

                reference_line = -selected_log10f-2
                maxval = np.max(selected_pxx_denlog-reference_line)
                dist_to_draw_to = maxval+0.5

                powerplot_log_log_1f_line.setData([0,3],[-2+dist_to_draw_to,-5+dist_to_draw_to])

                powerplot_log_log_curve.setData( log10f, power_log10f )

        if plot_configuration[p.SHOW_STATE_MONITOR] and clocktime > plot_configuration[
            p.STATE_MONITOR_DISPLAY_WINDOW_MS] and crt_data['state_monitor_matrix'] != None:
            for i in xrange(np.shape(crt_data['state_monitor_matrix'])[0]):
                # if np.shape(crt_data['state_monitor_matrix'])[0]>=30:
                #     import pdb
                #     pdb.set_trace()
                neuron_monitor_curves_array[i].setData(
                    np.arange(max(0, clocktime - plot_configuration[p.STATE_MONITOR_DISPLAY_WINDOW_MS]), clocktime),
                    crt_data['state_monitor_matrix'][i, :])
                neuron_monitor_curves_array[i].show()
            for i in xrange(np.shape(crt_data['state_monitor_matrix'])[0],50):
                neuron_monitor_curves_array[i].hide()

    timer = QtCore.QTimer()
    timer.timeout.connect(update)
    timer.start(10)

    app.exec_()
    print "finishedexec"


class Plot_data:
    def __init__(self, plot_configuration, simulation_configuration, types, excitatory_indexes_sorted,\
                 inhibitory_indexes_sorted,activity_variable_name='v',activity_variable_unit=1*mV):
        self.data_lock = threading.Lock()
        self.speed_lock = threading.Lock()
        self.neuron_indexes_lock = threading.Lock()

        self.plot_configuration = plot_configuration
        self.simulation_configuration = simulation_configuration
        self.types = types
        self.activity_variable_name = activity_variable_name
        self.activity_variable_unit = activity_variable_unit

        self.excitatory_indexes_sorted = excitatory_indexes_sorted
        self.inhibitory_indexes_sorted = inhibitory_indexes_sorted

        self.activity_history = np.zeros(
            (simulation_configuration[p.NETWORK_WIDTH], simulation_configuration[p.NETWORK_WIDTH])) - \
                                plot_configuration[p.SPIKES_INTEGRATION_WINDOW_MS]

        self.clocktime = 0

        self.types = types
        self.exc_spikes_data = np.zeros(plot_configuration[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS])
        self.inh_spikes_data = np.zeros(plot_configuration[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS])

        self.grid_data = []
        self.grid_data2 = []
        self.state_monitor_matrix = None
        self.u_exc_exc_matrix=None
        self.x_exc_exc_matrix=None

        self.frequency_changed = True
        self.speed = plot_configuration[p.PLOT_SPEED]

        self.frequency_values = None
        self.frequency_power = None

        number_of_neurons_to_state_monitor = plot_configuration[p.NUMBER_NEURONS_TO_MONITOR]
        self.state_monitor_neuron_indexes = np.zeros(number_of_neurons_to_state_monitor)

        for i in xrange(number_of_neurons_to_state_monitor):
            self.state_monitor_neuron_indexes[i] = i

    def calculate_new_plot_data(self, input_data):
        plot_p = self.plot_configuration
        sim_p = self.simulation_configuration

        excitatory_indexes_sorted = self.excitatory_indexes_sorted
        inhibitory_indexes_sorted = self.inhibitory_indexes_sorted

        clocktime = input_data['clocktime']

        time_ms = int(clocktime / ms)

        # import pdb
        # pdb.set_trace()

        population_monitor_exc = input_data['population_monitor_exc']
        population_monitor_inh = input_data['population_monitor_inh']
        num_excitatory = input_data['num_excitatory']
        num_inhibitory = input_data['num_inhibitory']

        current_grid_activation = input_data['current_grid_activation']

        spikes_integration_window = plot_p[p.SPIKES_INTEGRATION_WINDOW_MS]
        spike_evolution_display_window = plot_p[p.SPIKE_EVOLUTION_DISPLAY_WINDOW_MS]
        state_monitor_display_window = plot_p[p.STATE_MONITOR_DISPLAY_WINDOW_MS]
        power_plot_interval = plot_p[p.POWER_PLOT_INTERVAL_MS]

        membrane_potential_matrix = np.reshape(current_grid_activation,
                                               (sim_p[p.NETWORK_WIDTH], sim_p[p.NETWORK_WIDTH]))

        # [exc_spikes, inh_spikes] = get_separated_excinh_spike_times_interval(s_mon, self.types, max(0,
        #                                                                                             time_ms - spike_evolution_display_window),
        #                                                                      time_ms)

        #TODO: ugly hack, shouldn't be int here

        crt_excitatory_spikes = population_monitor_exc.rate[int(time_ms-1)]/Hz*num_excitatory*0.001
        crt_inhibitory_spikes = population_monitor_inh.rate[int(time_ms-1)]/Hz*num_inhibitory*0.001

        all_excitatory_spikes = population_monitor_exc.rate/Hz*num_excitatory*0.001
        all_inhibitory_spikes = population_monitor_inh.rate/Hz*num_inhibitory*0.001
        all_spikes = all_excitatory_spikes + all_inhibitory_spikes
        
        if time_ms < spike_evolution_display_window:
            self.exc_spikes_data[time_ms]=crt_excitatory_spikes
            self.inh_spikes_data[time_ms]=crt_inhibitory_spikes
        else:
            self.exc_spikes_data[0:-1]=self.exc_spikes_data[1:]
            self.inh_spikes_data[0:-1]=self.inh_spikes_data[1:]

            self.exc_spikes_data[spike_evolution_display_window-1]=crt_excitatory_spikes
            self.inh_spikes_data[spike_evolution_display_window-1]=crt_inhibitory_spikes

        self.activity_history = self.activity_history - plot_p[p.DATA_UPDATE_INTERVAL_MS]
        indexes_min = np.where(self.activity_history < -spikes_integration_window)
        self.activity_history[indexes_min] = -spikes_integration_window

        #here build a matrix with 1's for all neurons that just spiked, and use that to reset the activity history matrix for those neurons
        exc_lastspike=input_data['exc_lastspikes']
        inh_lastspike=input_data['inh_lastspikes']
        exc_spiked_last_timestep = (exc_lastspike==clocktime)
        inh_spiked_last_timestep = (inh_lastspike==clocktime)
        neuron_spikes_vector = np.zeros(sim_p[p.NETWORK_WIDTH]*sim_p[p.NETWORK_WIDTH])
        if sum(exc_spiked_last_timestep)>0:
            neuronindex_spikedlast_timestep_excitatory = excitatory_indexes_sorted[exc_spiked_last_timestep]
            neuron_spikes_vector[neuronindex_spikedlast_timestep_excitatory]=1
        if sum(exc_spiked_last_timestep)>0:
            neuronindex_spikedlast_timestep_inhibitory = inhibitory_indexes_sorted[inh_spiked_last_timestep]
            neuron_spikes_vector[neuronindex_spikedlast_timestep_inhibitory]=1
        neuron_spikes_matrix = np.reshape(neuron_spikes_vector, (sim_p[p.NETWORK_WIDTH], sim_p[p.NETWORK_WIDTH]))
        indexes_spike = np.where(neuron_spikes_matrix == 1)
        self.activity_history[indexes_spike] = 0

        if plot_p[p.SHOW_STATE_MONITOR] and time_ms > state_monitor_display_window:
            state_monitor = input_data['state_monitor']
            state_monitor_neuron_indexes = self.get_statemon_neuron_indexes()
            number_of_neurons_to_state_monitor = len(state_monitor_neuron_indexes)

            #this doesn't work if state monitor time is bigger than 1, or if there are breaks in between
            assert state_monitor.t[1]/ms-state_monitor.t[0]/ms==1.

            selected_indexes_state_monitor = np.arange(time_ms-state_monitor_display_window,time_ms).astype(int)

            state_monitor_matrix = self.get_statemon_matrix()

            if state_monitor_matrix == None:
                print "first got in here and initialised matrix"
                state_monitor_matrix = np.zeros((number_of_neurons_to_state_monitor, state_monitor_display_window))
                if len(selected_indexes_state_monitor) >= state_monitor_display_window:
                    # self.state_monitor_matrix =  getattr(state_monitor,self.activity_variable_name)[state_monitor_neuron_indexes, selected_indexes_state_monitor] / self.activity_variable_unit
                    for i in xrange(number_of_neurons_to_state_monitor):
                        state_monitor_matrix[i, :] = getattr(state_monitor,self.activity_variable_name)[state_monitor_neuron_indexes[i], selected_indexes_state_monitor] / self.activity_variable_unit
            else:
                #TODO: ugly hack, should fix synchronization problem between the 2.
                # print np.shape(state_monitor_matrix)[0],len(state_monitor_neuron_indexes)
                if(np.shape(state_monitor_matrix)[0] == len(state_monitor_neuron_indexes)):
                    state_monitor_matrix[:,np.arange(0,state_monitor_display_window-1)] = state_monitor_matrix[:,np.arange(1,state_monitor_display_window)]
                    state_monitor_matrix[:,state_monitor_display_window-1] = getattr(state_monitor,self.activity_variable_name)[state_monitor_neuron_indexes, selected_indexes_state_monitor[len(selected_indexes_state_monitor)-1]] / self.activity_variable_unit

            self.set_statemon_matrix(state_monitor_matrix)

            # if len(selected_indexes_state_monitor) >= state_monitor_display_window:
            #     # self.state_monitor_matrix =  getattr(state_monitor,self.activity_variable_name)[state_monitor_neuron_indexes, selected_indexes_state_monitor] / self.activity_variable_unit
            #     for i in xrange(number_of_neurons_to_state_monitor):
            #         self.state_monitor_matrix[i, :] = getattr(state_monitor,self.activity_variable_name)[state_monitor_neuron_indexes[i], selected_indexes_state_monitor] / self.activity_variable_unit

        if plot_p[p.SHOW_POWER_PLOT] and time_ms % power_plot_interval == 0 and time_ms > 0:
            self.frequency_values, self.frequency_power = get_welch(all_spikes,resolution=min(time_ms,26000))
            self.set_data_frequency(frequency_values=self.frequency_values, frequency_power=self.frequency_power)
        
        if plot_p[p.SHOW_U_EXC_EXC_SUM_PLOT]:
            u_exc_exc = input_data['u_exc_exc']
            print "u_exc_exc",max(u_exc_exc)
            u_exc_exc_matrix = np.reshape(u_exc_exc,
                                               (sim_p[p.NETWORK_WIDTH], sim_p[p.NETWORK_WIDTH]))
            self.set_u_exc_exc_matrix(u_exc_exc_matrix)
        
        if plot_p[p.SHOW_X_EXC_EXC_SUM_PLOT]:
            x_exc_exc = input_data['x_exc_exc']
            print "x exc exc",max(x_exc_exc)
            x_exc_exc_matrix = np.reshape(x_exc_exc,
                                               (sim_p[p.NETWORK_WIDTH], sim_p[p.NETWORK_WIDTH]))
            self.set_x_exc_exc_matrix(x_exc_exc_matrix)        
        #print np.min(membrane_potential_matrix)

        self.set_data(time_ms, self.exc_spikes_data, self.inh_spikes_data, self.activity_history, membrane_potential_matrix,
                      state_monitor_matrix=self.state_monitor_matrix,
                      state_monitor_neuron_indexes=self.state_monitor_neuron_indexes)

    def set_data(self, clocktime, exc_spikes_data, inh_spikes_data, grid_data, grid_data2,
                 state_monitor_matrix=None, state_monitor_neuron_indexes=None, ):
        self.data_lock.acquire()
        self.neuron_indexes_lock.acquire()

        self.clocktime = clocktime
        self.exc_spikes_data = exc_spikes_data
        self.inh_spikes_data = inh_spikes_data
        self.grid_data = grid_data
        self.grid_data2 = grid_data2
        self.state_monitor_matrix = state_monitor_matrix
        self.state_monitor_neuron_indexes = state_monitor_neuron_indexes

        self.data_lock.release()
        self.neuron_indexes_lock.release()

    def get_statemon_matrix(self):
        self.neuron_indexes_lock.acquire()
        statemon_matrix = self.state_monitor_matrix
        self.neuron_indexes_lock.release()
        return statemon_matrix

    def set_statemon_matrix(self,statemon_matrix):
        self.neuron_indexes_lock.acquire()
        self.state_monitor_matrix = statemon_matrix
        self.neuron_indexes_lock.release()
    
    def get_u_exc_exc_matrix(self):
        self.neuron_indexes_lock.acquire()
        u_exc_exc_matrix = self.u_exc_exc_matrix
        self.neuron_indexes_lock.release()
        return u_exc_exc_matrix

    def set_u_exc_exc_matrix(self,u_exc_exc_matrix):
        self.neuron_indexes_lock.acquire()
        self.u_exc_exc_matrix = u_exc_exc_matrix
        self.neuron_indexes_lock.release()

    def get_x_exc_exc_matrix(self):
        self.neuron_indexes_lock.acquire()
        x_exc_exc_matrix = self.x_exc_exc_matrix
        self.neuron_indexes_lock.release()
        return x_exc_exc_matrix

    def set_x_exc_exc_matrix(self,x_exc_exc_matrix):
        self.neuron_indexes_lock.acquire()
        self.x_exc_exc_matrix = x_exc_exc_matrix
        self.neuron_indexes_lock.release()

    def set_statemon_neuron_indexes(self,neuron_indexes):
        self.neuron_indexes_lock.acquire()
        self.state_monitor_neuron_indexes = neuron_indexes
        #reset matrix
        self.state_monitor_matrix = None
        self.neuron_indexes_lock.release()

    def get_statemon_neuron_indexes(self):
        self.neuron_indexes_lock.acquire()
        state_monitor_neuron_indexes = self.state_monitor_neuron_indexes.astype(int)
        self.neuron_indexes_lock.release()
        return state_monitor_neuron_indexes

    def set_central_neuron(self,x,y):
        self.central_x = y
        self.central_y = x

        grid_width = 50
        local_range = 3

        max_length_top = x - max(x-local_range,0)
        max_length_bottom = min(x+local_range,grid_width)-x

        max_length_left = y - max(y-local_range,0)
        max_length_right = min(y+local_range,grid_width)-y

        square_length = (max_length_bottom+max_length_top)*(max_length_right+max_length_left)

        neuron_indexes = np.zeros(square_length)

        count=0
        for i in xrange(x-max_length_top,x+max_length_bottom):
            for j in xrange(y-max_length_left,y+max_length_right):
                neuron_indexes[count]=i*grid_width+j
                count+=1

        self.set_statemon_neuron_indexes(neuron_indexes)

    def set_data_frequency(self, frequency_values=None, frequency_power=None):
        self.data_lock.acquire()

        self.frequency_values = frequency_values
        self.frequency_power = frequency_power

        self.frequency_changed = True

        self.data_lock.release()

    def get_data(self):
        self.data_lock.acquire()
        self.neuron_indexes_lock.acquire()

        exc_spikes_data = self.exc_spikes_data
        inh_spikes_data = self.inh_spikes_data
        grid_data = self.grid_data
        grid_data2 = self.grid_data2
        clocktime = self.clocktime
        state_monitor_matrix = self.state_monitor_matrix
        state_monitor_neuron_indexes = self.state_monitor_neuron_indexes
        types = self.types
        frequency_values = self.frequency_values
        frequency_power = self.frequency_power
        frequency_changed = self.frequency_changed
        self.frequency_changed = False
        self.data_lock.release()
        self.neuron_indexes_lock.release()

        my_dict = {'types': types, 'clocktime': clocktime, 'exc_spikes_data': exc_spikes_data,
                   'inh_spikes_data': inh_spikes_data,
                   'grid_data': grid_data, 'grid_data2': grid_data2, 'state_monitor_matrix': state_monitor_matrix,
                   'state_monitor_neuron_indexes': state_monitor_neuron_indexes,
                   'frequency_values': frequency_values, 'frequency_power': frequency_power,
                   'frequency_changed': frequency_changed,'simulation_configuration':self.simulation_configuration}

        return my_dict

    def set_speed(self, speed):
        self.speed_lock.acquire()
        self.speed = speed
        self.speed_lock.release()

    def get_speed(self):
        self.speed_lock.acquire()
        speed = self.speed
        self.speed_lock.release()

        return speed

    def get_types(self):
        return self.types


class Live_plotting_window:
    def __init__(self, plot_data):
        threadWindow = threading.Thread(target=(run_app), args=(plot_data,))
        threadWindow.start()