#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from random import random
from cros_model.utils import properties

#SIMULATION PARAMETERS
SIMULATION_TYPE = "Simulation Type"
RANDOM_SEED = "Random seed"
SIMULATION_LENGTH = "Simulation Length"
TOTAL_SIMULATION_LENGTH = "Total simulation length"
RUN_NUMBER = "Run Number"

#NETWORK PARAMETERS
NUMBER_NEURONS = "Number Neurons"
NETWORK_WIDTH = "Network Width"
EXCITATORY_CONNECTIVITY = "Excitatory Connectivity"
INHIBITORY_CONNECTIVITY = "Inhibitory Connectivity"
FRACTION_EXCITATORY = "fracEx"
W_EX_EX = "wExEx"
W_EX_IN = "wExIn"
W_IN_IN = "wInIn"
W_IN_EX = "wInEx"
PLASTICITY = "Plasticity"
NETWORK_TO_LOAD_PATH = "Network to load path"
NETWORK_TO_LOAD_PATH_FORMAT = "Network to load path format"
NETWORK_TO_LOAD_PATH_INPUT_PARAMETERS = "Network to load path input parameters"

#ORIGINAL CROS MODEL PARAMETERS
EXC_DECAY = "excDecay"
INH_DECAY = "inhDecay"
INPUT_DECAY_EX = "inputDecay Ex"
INPUT_DECAY_IN = "inputDecay In"
LOCAL_RANGE_EX = "Local Range Ex"
LOCAL_RANGE_IN = "Local Range In"
P_RESET_EXC = "P reset excitatory"
P_RESET_INH = "P reset inhibitory"
BACKGROUND_P_EXC = "Excitatory Background Rate"
BACKGROUND_P_INH = "Inhibitory Background Rate"

#NEUROMOD GAIN PARAMETERS
EE_GAIN = "eeGain"
EI_GAIN = "eiGain"
IE_GAIN = "ieGain"
II_GAIN = "iiGain"

SIMULATION_PARAMETERS = "Simulation parameters"
STANDALONE_MODE = "Standalone mode"
VERBOSE = "Verbose"
OUTPUT = "Output"
FITNESS_FUNCTION = "Fitness function"

OUTPUT_DIR = "Output dir"
OUTPUT_NETWORK = "Output network"
OUTPUT_ACTIVITY = "Output activity"
OUTPUT_DIR_PATH_FORMAT = "Output dir path format"
OUTPUT_DIR_PATH_PARAMETERS = "Output dir path parameters"

EXTERNAL_INPUT = "External input"
STIMULUS_SIZE = "Stimulus size"
WEIGHT_EXTERNAL_INPUT = "Weight external input"
STIMULUS_NEIGHBORHOOD_CLUSTER_SIZE = "Stimulus neighborhood cluster size"

SHOW_SIM_GUI = "Show sim GUI"

CONNECTIVITY_PROBABILITY = "Connectivity probability"

NETWORK_TYPE="Network type"
NETWORK_TYPE_SQUARE_NEIGHBORHOOD_EXPONENTIAL_DISTANCE = "Square neighborhood exponential distance"
NETWORK_TYPE_CIRCLE_NEIGHBORHOOD_EXPONENTIAL_DISTANCE = "Circle neighborhood exponential distance"
NETWORK_TYPE_CIRCLE_NEIGHBORHOOD_EXPONENTIAL_LINEAR_DISTANCE = "Circle neighborhood exponential/linear distance"
NETWORK_TYPE_RANDOM_CONNECTIVITY = "Random connectivity"
LINEAR_DISTANCE_RATIO = "Linear distance ratio"
TORUS_WRAP_AROUND = "Torus wrap around"
RANDOM_REWIRING_PERCENTAGE = "Random rewiring percentage"

SIMULATION_SCRIPT = "Simulation script"
RUN_COMMAND="Run command"
SHORT_RUNTIME_TIMEOUT="Short runtime timeout"
RUNTIME_TIMEOUT="Runtime timeout"

SPIKE_TRAIN_SUBSET_EXCITATORY = "Spike train subset excitatory"
SPIKE_TRAIN_SUBSET_INHIBITORY = "Spike train subset inhibitory"

RUNMODE = "Run mode"
RUNMODE_GUI = "gui"
RUNMODE_BATCH = "batch"
RUNMODE_SINGLE = "single"

TRACK_AVALANCHES = "Track avalanches"

STOP_POPULATION_RATE = "Stop if population rate too high"
STOP_PACEMAKER = "Stop if pacemaker found"

RUN_ANALYSIS_SCRIPT_NAME = "Run analysis script name"

TYPE_EXC=1
TYPE_INH=0

def get_default_parameters():
    default_parameters = {}

    default_parameters[STANDALONE_MODE]=True
    default_parameters[SIMULATION_LENGTH]=10

    default_parameters[VERBOSE]=True
    default_parameters[OUTPUT]=True
    default_parameters[OUTPUT_DIR]="original"
    default_parameters[OUTPUT_NETWORK]=False
    default_parameters[OUTPUT_ACTIVITY]=False
    default_parameters[RUN_NUMBER]=0
    default_parameters[PLASTICITY] = False
    default_parameters[EXTERNAL_INPUT] = False
    default_parameters[STIMULUS_SIZE]=0
    default_parameters[SHOW_SIM_GUI] = False
    default_parameters[RUNTIME_TIMEOUT]=500
    default_parameters[RUN_ANALYSIS_SCRIPT_NAME]="run_analysis"

    return default_parameters

def get_gridnetwork_parameters():
    gridnetwork_parameters = get_default_parameters()

    gridnetwork_parameters[FRACTION_EXCITATORY] = 0.75
    gridnetwork_parameters[NETWORK_WIDTH] = 50
    gridnetwork_parameters[NUMBER_NEURONS]= gridnetwork_parameters[NETWORK_WIDTH]*gridnetwork_parameters[NETWORK_WIDTH]
    gridnetwork_parameters[LOCAL_RANGE_EX]=3
    gridnetwork_parameters[LOCAL_RANGE_IN]=3
    gridnetwork_parameters[EXCITATORY_CONNECTIVITY]=0.5
    gridnetwork_parameters[INHIBITORY_CONNECTIVITY]=0.7
    gridnetwork_parameters[NETWORK_TYPE]=NETWORK_TYPE_CIRCLE_NEIGHBORHOOD_EXPONENTIAL_DISTANCE
    gridnetwork_parameters[TORUS_WRAP_AROUND]=False

    gridnetwork_parameters[STOP_PACEMAKER]=False
    gridnetwork_parameters[STOP_POPULATION_RATE]=False
    
    gridnetwork_parameters[RANDOM_REWIRING_PERCENTAGE]=0

    return gridnetwork_parameters

def get_cros_parameters():
    cros_parameters = get_gridnetwork_parameters()
    
    cros_parameters[INPUT_DECAY_EX]=9
    cros_parameters[INPUT_DECAY_IN]=9

    cros_parameters[EXC_DECAY]=6
    cros_parameters[INH_DECAY] = 12

    cros_parameters[P_RESET_EXC] = -2
    cros_parameters[P_RESET_INH] = -20
    cros_parameters[BACKGROUND_P_EXC] = 0.000001
    cros_parameters[BACKGROUND_P_INH] = 0
    cros_parameters[W_EX_EX] = (0.0085) # excitatory synaptic weight (voltage)
    cros_parameters[W_EX_IN] = (0.0085) # excitatory synaptic weight (voltage)
    cros_parameters[W_IN_EX] = (-0.569) # excitatory synaptic weight (voltage)
    cros_parameters[W_IN_IN] = (-2) # inhibitory synaptic weight

    cros_parameters[EE_GAIN]=1
    cros_parameters[EI_GAIN]=1
    cros_parameters[IE_GAIN]=1
    cros_parameters[II_GAIN]=1

    cros_parameters[NETWORK_TYPE]=NETWORK_TYPE_SQUARE_NEIGHBORHOOD_EXPONENTIAL_DISTANCE
    
    return cros_parameters


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

#removes additional (unused arguments from string), that is '#' and '*'
unused_arguments = ['#','*']
def cleanup_arguments(arguments_list):
    cleaned_arguments_list = []
    for i in xrange(len(arguments_list)):
        crt_argument = arguments_list[i]
        if len(crt_argument)==1 and crt_argument in unused_arguments:
            continue
        else:
            cleaned_arguments_list.append(crt_argument)

    return cleaned_arguments_list

def load_params_file(file_path,rest_of_arguments):
    new_params = properties.load_properties_file(file_path)

    cleaned_arguments = cleanup_arguments(rest_of_arguments)

    if len(rest_of_arguments)>0:
        assert SIMULATION_PARAMETERS in new_params.keys()
        simulation_parameters_vector=new_params[SIMULATION_PARAMETERS].split(",")
        assert len(simulation_parameters_vector)==len(cleaned_arguments)
        for i in xrange(len(simulation_parameters_vector)):
            property_value = cleaned_arguments[i].strip()
            if is_number(property_value):
                property_value = float(property_value)
                try:
                    property_value_int = int(property_value)
                    if property_value_int==property_value:
                        property_value = property_value_int
                except:
                    pass
            elif property_value == "True":
                property_value = True
            elif property_value == "False":
                property_value = False
            new_params[simulation_parameters_vector[i].strip()]=property_value

    if NETWORK_WIDTH in new_params.keys():
        new_params[NUMBER_NEURONS]=new_params[NETWORK_WIDTH]**2
    
    #TODO: find better place to move this code
    if RUNMODE in new_params.keys():
        if new_params[RUNMODE]== RUNMODE_GUI:
            new_params[SHOW_SIM_GUI]=True
        
        if new_params[RUNMODE] not in new_params[OUTPUT_DIR]:
            new_params[OUTPUT_DIR]=new_params[OUTPUT_DIR]+"_" +new_params[RUNMODE]
    return new_params

def print_parameters(parameters):
    print "#"*50
    for crt_key in sorted(parameters.keys()):
        crt_value = parameters[crt_key]
        print crt_key+"="+str(crt_value)
    print "#"*50

def get_random_seed():
    return int(round(random()*10000000000000))

def output_params(parameters,path):
    with open(path,"w") as f:
        for crt_key in sorted(parameters.keys()):
            crt_value = parameters[crt_key]
            f.write(crt_key+"="+str(crt_value)+"\n")

SAVE_FIRST_AVALANCHE_ACTIVITY="Save first avalanche activity"
SPIKES_INTEGRATION_WINDOW_MS = "Spikes integration window"
SPIKE_EVOLUTION_DISPLAY_WINDOW_MS = "Spike evolution display window"
STATE_MONITOR_DISPLAY_WINDOW_MS = "State monitor display window"
SHOW_STATE_MONITOR = "Show state monitor"
SHOW_HISTOGRAM_STATE = "Show histogram state"
POWER_PLOT_INTERVAL_MS = "Power plot interval"
SHOW_POWER_PLOT = "Show power plot"
SHOW_LOG_LOG_POWER_PLOT = "Show log log plot"
SHOW_U_EXC_EXC_SUM_PLOT = "Show u exc exc sum plot"
SHOW_X_EXC_EXC_SUM_PLOT = "Show x exc exc sum plot"
DATA_UPDATE_INTERVAL_MS = "Data update interval(ms)"
NUMBER_NEURONS_TO_MONITOR = "Number neurons to monitor"
PLOT_SPEED = "Plot speed"

def get_plotting_parameters():
    default_parameters = {}
    default_parameters[SAVE_FIRST_AVALANCHE_ACTIVITY] = True
    default_parameters[SPIKES_INTEGRATION_WINDOW_MS] = 80
    default_parameters[SPIKE_EVOLUTION_DISPLAY_WINDOW_MS] = 1000
    default_parameters[STATE_MONITOR_DISPLAY_WINDOW_MS] = 100
    default_parameters[POWER_PLOT_INTERVAL_MS] = 1000
    default_parameters[NUMBER_NEURONS_TO_MONITOR] = 20
    default_parameters[DATA_UPDATE_INTERVAL_MS] = 1

    default_parameters[SHOW_STATE_MONITOR] = False
    default_parameters[SHOW_HISTOGRAM_STATE] = True
    default_parameters[SHOW_POWER_PLOT] = True
    default_parameters[SHOW_LOG_LOG_POWER_PLOT]=True
    default_parameters[SHOW_U_EXC_EXC_SUM_PLOT]=False
    default_parameters[SHOW_X_EXC_EXC_SUM_PLOT]=False
    default_parameters[PLOT_SPEED]="high"

    return default_parameters

DFA_CALC_SMALLEST_WINDOW = "DFA calc smallest window"
DFA_CALC_LARGEST_WINDOW = "DFA calc largest window"
DFA_CALC_FREQUENCY_LOW = "DFA calc frequency low"
DFA_CALC_FREQUENCY_HIGH = "DFA calc frequency high"
DFA_CALC_WINDOW_OVERLAP = "DFA calc window overlap"
DFA_FIT_SMALLEST_WINDOW = "DFA fit smallest window"
DFA_FIT_LARGEST_WINDOW = "DFA fit largest window"
KAPPA_AVALANCHE_SIZE_MIN = "Kappa avalanche size min"
KAPPA_AVALANCHE_SIZE_MAX = "Kappa avalanche size max"
KAPPA_BINS = "Kappa bins"
PLOT_DFA="Plot DFA"
PLOT_KAPPA="Plot Kappa"

def get_analysis_parameters():
    analysis_parameters = {}

    analysis_parameters[DFA_CALC_SMALLEST_WINDOW]=1
    analysis_parameters[DFA_CALC_LARGEST_WINDOW]=120
    analysis_parameters[DFA_CALC_FREQUENCY_LOW]=8
    analysis_parameters[DFA_CALC_FREQUENCY_HIGH]=16
    analysis_parameters[DFA_FIT_SMALLEST_WINDOW]=5
    analysis_parameters[DFA_FIT_LARGEST_WINDOW]=30
    analysis_parameters[KAPPA_AVALANCHE_SIZE_MAX]=1
    analysis_parameters[KAPPA_AVALANCHE_SIZE_MAX]=10000
    analysis_parameters[KAPPA_BINS]=12

    analysis_parameters[PLOT_DFA]=True
    analysis_parameters[PLOT_KAPPA]=False

    return analysis_parameters
