#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from random import random
import os

import cros_model.params as p
from cros_model.configuration import configuration as conf
from cros_model.simulation import simulation_utils

rand_value = str(int(round(random()*10000000000000)))
crt_dir = os.path.dirname(__file__)+os.path.sep

configuration = conf.get_configuration()
simulation_param_files_path = configuration[conf.SIMULATION_PARAM_FILES_PATH]

def run_for_network_params(parameters):
    parameters_crt = parameters.copy()
    
    simulation_utils.run_simulation(parameters_crt,parameters_crt[p.SIMULATION_SCRIPT],rand_value,parameters_crt[p.RUNTIME_TIMEOUT])
    return

########################################## PARAMETERS LOAD
parameters = p.get_cros_parameters()
#set random seed
random_seed = p.get_random_seed()
parameters[p.RANDOM_SEED]=random_seed
parameters[p.RUN_COMMAND]='python '+' '.join(sys.argv)

#override simulation parameters with parameters from the commandline
rest_of_arguments = sys.argv[2:]
params_file_path = simulation_utils.get_current_or_old_path(simulation_param_files_path, sys.argv[1])
params_to_override = p.load_params_file(params_file_path,rest_of_arguments)
for crt_key in params_to_override.keys():
    parameters[crt_key]=params_to_override[crt_key]

# one way to load the network is by specifying the path format to the network. this is especially useful when running phase space
# another way is just specify the path directly to that network structure, which is not done here, but in the simulation parameter file
if p.NETWORK_TO_LOAD_PATH_FORMAT in parameters.keys():
    network_path_parameters_keys = parameters[p.NETWORK_TO_LOAD_PATH_INPUT_PARAMETERS].split(",")
    network_path_parameters_values = []
    for parameter_key in network_path_parameters_keys:
        network_path_parameters_values.append(parameters[parameter_key.strip()])
    
    parameters[p.NETWORK_TO_LOAD_PATH]=(configuration[conf.NETWORKS_FILE_PATH] + parameters[p.NETWORK_TO_LOAD_PATH_FORMAT]+'/SimulationData.h5') \
                  % tuple(network_path_parameters_values)

########################################## RUN

run_for_network_params(parameters)