#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from brian2 import *
import cros_model.network as network
from cros_model.output import output
import shutil
from time import time
import cros_model.params as p
import os
import random
from subprocess import call
import numpy_indexed as npi

########################################## HERE WE JUST INITIALIZE THE SIMULATION, WITHOUT RUNNING IT, IN ORDER TO GENERATE THE NETWORK STRUCTURE

params_file_path = sys.argv[1]
output_file_path = sys.argv[2]

parameters = p.load_params_file(params_file_path,"")

random.seed(parameters[p.RANDOM_SEED])

simulation_output = output.Output(parameters)
simulation_output.write_params()

########################################## CREATE NETWORK

#initialize network structure before standalone cpp (code generator cannot handle cpp)
num_neurons=int(parameters[p.NUMBER_NEURONS])
types=np.zeros(num_neurons)

num_excitatory = int(round(parameters[p.FRACTION_EXCITATORY]*num_neurons))
num_inhibitory = num_neurons-num_excitatory

if not p.NETWORK_TO_LOAD_PATH in parameters.keys():
    all_indexes = np.array(range(0,num_neurons))
    np.random.shuffle(all_indexes)
    types[all_indexes[0:num_excitatory]]=1

    excitatory_indexes = all_indexes[0:num_excitatory]
    inhibitory_indexes = all_indexes[num_excitatory:num_neurons]
    excitatory_indexes_sorted = np.sort(all_indexes[0:num_excitatory])
    inhibitory_indexes_sorted = np.sort(all_indexes[num_excitatory:num_neurons])

    [ee_from_to,ei_from_to,ie_from_to,ii_from_to]= network.get_network(num_neurons,types,parameters)
        
    wEE = parameters[p.W_EX_EX]
    wEI = parameters[p.W_EX_IN]
    wIE = parameters[p.W_IN_EX]
    wII = parameters[p.W_IN_IN]
    
    if parameters[p.EXTERNAL_INPUT]:
        stimulus_size = parameters[p.STIMULUS_SIZE]
        stimulus_input_from_to = np.column_stack( (np.arange(stimulus_size),np.random.choice(excitatory_indexes,size = stimulus_size,replace = False)) )
        w_external_exc = 0.0085
    else:
        stimulus_size = 0
        stimulus_input_from_to = np.empty([0,2])
        w_external_exc = 0
else:
    print "loading network form path"
    #TODO: add in some checks for network size. not needed now
    [types,types_stimulus,ee_from_to,ei_from_to,ie_from_to,ii_from_to,stimulus_input_from_to,wEE,wEI,wIE,wII,w_external_exc]=output.get_network_arrays_from_path(parameters[p.NETWORK_TO_LOAD_PATH])
    
    stimulus_size = len(types_stimulus)
    
    excitatory_indexes = np.where(types==1)[0]
    inhibitory_indexes = np.where(types==0)[0]
    
    excitatory_indexes_sorted = np.sort(excitatory_indexes)
    inhibitory_indexes_sorted = np.sort(inhibitory_indexes)

ee_from = ee_from_to[:,0]
ee_to = ee_from_to[:,1]
ei_from = ei_from_to[:,0]
ei_to = ei_from_to[:,1]
ie_from = ie_from_to[:,0]
ie_to = ie_from_to[:,1]
ii_from = ii_from_to[:,0]
ii_to = ii_from_to[:,1]
stimulus_input_from = stimulus_input_from_to[:,0]
stimulus_input_to = stimulus_input_from_to[:,1]

ee_from_idx = npi.indices(excitatory_indexes_sorted, ee_from)
ee_to_idx = npi.indices(excitatory_indexes_sorted, ee_to)
ei_from_idx = npi.indices(excitatory_indexes_sorted, ei_from)
ei_to_idx = npi.indices(inhibitory_indexes_sorted, ei_to)
ie_from_idx = npi.indices(inhibitory_indexes_sorted, ie_from)
ie_to_idx = npi.indices(excitatory_indexes_sorted, ie_to)
ii_from_idx = npi.indices(inhibitory_indexes_sorted, ii_from)
ii_to_idx = npi.indices(inhibitory_indexes_sorted, ii_to)
stimulus_input_from_idx = stimulus_input_from #this doesn't change
stimulus_input_to_idx = npi.indices(excitatory_indexes_sorted,stimulus_input_to)

background_p_exc=parameters[p.BACKGROUND_P_EXC]
background_p_inh=parameters[p.BACKGROUND_P_INH]
taup_exc=parameters[p.EXC_DECAY]*ms
taup_inh=parameters[p.INH_DECAY]*ms
taui_exc=parameters[p.INPUT_DECAY_EX]*ms
taui_inh=parameters[p.INPUT_DECAY_IN]*ms
P_reset_exc=parameters[p.P_RESET_EXC]
P_reset_inh=parameters[p.P_RESET_INH]
network_size=parameters[p.NETWORK_WIDTH]
num_neurons=parameters[p.NUMBER_NEURONS]

tauI_e=parameters[p.INPUT_DECAY_EX]*ms
tauI_i=parameters[p.INPUT_DECAY_IN]*ms

########################################## MODEL INITIALIZATION

#assign some of the parameters here in the local scope to be used in the model
defaultclock.dt = 1*ms
tau_s=1*ms
tau_clock_mult = defaultclock.dt/ms

eqs = '''
dI/dt = (S/tau_s + (-(I)/taui) ) * tau_clock_mult : 1
dp/dt = ((I)/tau_s + (background_p - (p)) /taup) * tau_clock_mult : 1
dS/dt = -S/tau_s * tau_clock_mult : 1
background_p : 1
reset_p : 1
taup : second
'''

#assign some of the parameters here in the local scope to be used in the model
defaultclock.dt = 1*ms
tau_s=1*ms
tau_clock_mult = defaultclock.dt/ms

eqs = '''
dIe/dt = Ae/tau_s + (-Ie/tauI_e)  : 1
dIi/dt = Ai/tau_s + (-Ii/tauI_i)  : 1
dp/dt = (Ii+Ie)/tau_s + (background_p - p) /taup : 1
dAe/dt = -Ae/tau_s  : 1
dAi/dt = -Ai/tau_s  : 1
background_p : 1
reset_p : 1
taup : second
'''

neurongroup_exc = NeuronGroup(num_excitatory, eqs, threshold='rand()<=p', reset='p = reset_p',refractory=0*ms,method='euler')
neurongroup_exc.p=background_p_exc
neurongroup_exc.background_p= background_p_exc
neurongroup_exc.reset_p=P_reset_exc
neurongroup_exc.taup=taup_exc

neurongroup_inh = NeuronGroup(num_inhibitory, eqs, threshold='rand()<=p', reset='p = reset_p',refractory=0*ms,method='euler')
neurongroup_inh.p=background_p_inh
neurongroup_inh.background_p= background_p_inh
neurongroup_inh.reset_p=P_reset_inh
neurongroup_inh.taup=taup_inh

net = Network()

########################################## ASSIGN VARIABLES

#define neuron types

net.add(neurongroup_exc)
net.add(neurongroup_inh)

ee_connections = np.arange(0,len(ee_from))
ei_connections = np.arange(0,len(ei_from))

interval_weight_update = parameters[p.SIMULATION_LENGTH]/4

########################################## NETWORK MODEL

ee_gain = parameters[p.EE_GAIN]
ei_gain = parameters[p.EI_GAIN]
ie_gain = parameters[p.IE_GAIN]
ii_gain = parameters[p.II_GAIN]

neuron_model = 'w :1'
pre_model_ee = 'Ae+=ee_gain*w'
pre_model_ei = 'Ae+=ei_gain*w'
pre_model_ie = 'Ai+=ie_gain*w'
pre_model_ii = 'Ai+=ii_gain*w'

#separated this one because I do not to apply gain to it
pre_model_external_input = 'Ae+=w'
Cee = Synapses(neurongroup_exc, neurongroup_exc,neuron_model, pre=pre_model_ee,method='euler')
Cei = Synapses(neurongroup_exc, neurongroup_inh,neuron_model, pre=pre_model_ei,method='euler')
Cie = Synapses(neurongroup_inh, neurongroup_exc,neuron_model, pre=pre_model_ie,method='euler')
Cii = Synapses(neurongroup_inh, neurongroup_inh,neuron_model, pre=pre_model_ii,method='euler')

Cee.connect(i=ee_from_idx,j=ee_to_idx)
Cei.connect(i=ei_from_idx,j=ei_to_idx)
Cie.connect(i=ie_from_idx,j=ie_to_idx)
Cii.connect(i=ii_from_idx,j=ii_to_idx)

Cee.w = wEE
Cei.w = wEI
Cie.w = wIE
Cii.w = wII

Cse = []

net.add(Cee)
net.add(Cei)
net.add(Cie)
net.add(Cii)

if parameters[p.OUTPUT_NETWORK]:
    simulation_output.write_network_structure(excitatory_indexes_sorted,inhibitory_indexes_sorted,types,Cee,Cei,Cie,Cii,Cse,stimulus_size)

if parameters[p.STANDALONE_MODE]:
    generated_code_directory = simulation_output.get_output_dir_path()+'output'
    if os.path.isdir(generated_code_directory):
        shutil.rmtree(generated_code_directory)