#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess
import shlex
from threading import Timer
import shutil
import os
import sys
import psutil

import cros_model.params as p
from cros_model.configuration import configuration as conf
import cros_model.output.output as output
import cros_model.utils.os_utils as os_utils

def run(cmd, timeout_sec, output_path):

    def kill_proc(p):
        for child in p.children(recursive=True):  # or parent.children() for recursive=False
            child.kill()            
        p.kill()

    #TODO: quick fix, verify why this doesn't work on linux
    if os_utils.is_windows():
        proc = subprocess.Popen(shlex.split(cmd),stdout=subprocess.PIPE, shell=True)
        timer = Timer(timeout_sec, kill_proc, [proc])
        try:
            timer.start()

            with open(output_path+"output.txt",'w') as f:
                while proc.poll() is None:
                    output = proc.stdout.readline()
                    print output,
                    f.write(output)
        except:
            print "Unexpected error:", sys.exc_info()[0]
        finally:
            timer.cancel()
    else:

        proc = subprocess.Popen("{}".format(cmd),stdout=subprocess.PIPE,shell=True)
        p = psutil.Process(proc.pid)
                
        timer = Timer(timeout_sec, kill_proc, [p])
        try:
            timer.start()

            with open(output_path+"output.txt",'w') as f:
                while proc.poll() is None:
                    output = proc.stdout.readline()
                    print output,
                    f.write(output)
        except:
            print "Unexpected error:", sys.exc_info()[0]
        finally:
            timer.cancel()
            
        generated_code_directory = output_path+'output/'
        if os.path.isdir(generated_code_directory):
            shutil.rmtree(generated_code_directory)

def remove_file_if_exists(file_path):
    try:
        os.remove(file_path)
    except OSError:
        pass

def remove_dir_if_exists(dir_path):
    if os.path.isdir(dir_path):
        shutil.rmtree(dir_path)

def run_simulation(parameters,runfile_path,rand_value,run_max_duration=500):
    configuration = conf.get_configuration()

    #a parameters file will be created at a temporary location to be used as input for the simulation
    tmp_dir = configuration[conf.TMP_FILE_PATH]
    tmp_output = tmp_dir+"Output"+rand_value+".txt"
    tmp_params_file =tmp_dir+"Parameters_"+str(rand_value)+".params"
    p.output_params(parameters,tmp_params_file)

    #create output path if not exists
    output_path = output.get_output_directory_name(parameters)
    if not os.path.exists(output_path):
        os.makedirs(output_path)

    #run simulation
    configuration = conf.get_configuration()
    script_pathway = get_current_or_old_path(configuration[conf.SIMULATIONS_PATH],runfile_path)
    
    print "script pathway",script_pathway

    command_array=['python',"\""+script_pathway+"\"","\""+tmp_params_file+"\"","\""+tmp_output+"\""]
    cmd = " ".join(command_array)
    run(cmd,run_max_duration,output_path)

    #remove temporary parameters file
    remove_file_if_exists(tmp_params_file)

    return

def get_current_or_old_path(file_path,file_name):
    if os.path.isfile(file_path+file_name):
        full_path=file_path+file_name
    elif os.path.isfile(file_path+"old/"+file_name):
        full_path = file_path+"old/"+file_name
    else:
        raise IOError("could not find file at either "+file_path+file_name+" or "+file_path+"old/"+file_name)

    return full_path