#     Originally created by Arthur-Ervin Avramiea (2018), a.e.avramiea@vu.nl

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

import math
import random
import numpy as np

def nextTime(rateParameter):
    return -math.log(1.0 - random.random()) / rateParameter

def getPoissonSequenceOfEvents(averageTimeDifference,totalTimesteps):
    eventTimes = []
    crtTime = 0

    exceededAllottedTime = False
    while not exceededAllottedTime:
        newTimeDifference = nextTime(1.0/averageTimeDifference)

        if crtTime+newTimeDifference>totalTimesteps:
            exceededAllottedTime = True
        else:
            crtTime+=newTimeDifference
            eventTimes.append(int(crtTime))

    return sorted(list(set(eventTimes)))

def getNormallyDistributedCappedSample(sampleMean,sampleSD,sampleSize,sampleMinRatioFromMean,hasSampleMaxLimit=False,sampleMaxLimit=1):
    random_sample = np.random.normal(sampleMean,sampleSD,sampleSize)
    submin = np.where(random_sample<sampleMean*sampleMinRatioFromMean)
    random_sample[submin] = sampleMean*sampleMinRatioFromMean
    
    if hasSampleMaxLimit:
        overmax = np.where(random_sample>sampleMaxLimit)
        random_sample[overmax]=sampleMaxLimit
#     import pdb
#     pdb.set_trace()
    
    return random_sample