% This file is released under the terms of the GNU General Public License,
% version 3. See http://www.gnu.org/licenses/gpl.html

function [DFA_x, DFA_y] = calculateDfaPoints(data, samplingRate, CalcInterval, DFA_Overlap, hpass, lpass, finalPassBandwidth)
    persistent ham
    if isempty(ham)
        ham = hamming(2^13);
    end

    if nargin == 6
        uppass = hpass;
        lowpass = lpass;
    else
        [a,b] = pwelch(data,ham,[],[],1000);
        startind = find(b>hpass,1,'first');
        endind = find(b>lpass,1,'first');
        [~, maxind] = max(a(startind:endind));
        maxind = maxind + startind - 1;
        uppass = (-finalPassBandwidth + sqrt(finalPassBandwidth^2 + 4*b(maxind)^2))/2;
        lowpass = uppass + finalPassBandwidth;
    end

    [AmplitudeEnvelope, AmplitudeEnvelopeInfo] = nbt_GetAmplitudeEnvelope(data, samplingRate, uppass, lowpass, 2/uppass);
    intlenght = (CalcInterval(2)-CalcInterval(1))/4;
    

    try
        [DFA_x,DFA_y, ~] = nbt_doDFA(AmplitudeEnvelope, samplingRate, [CalcInterval(1)+intlenght, CalcInterval(2)-intlenght], CalcInterval, DFA_Overlap);
        DFA_x = DFA_x';
    catch error
        display(error) 
        warning('Could not calculate DFA');
        DFA_x = [];
        DFA_y = [];
    end
end
