library(plyr)

library(scales)
library(ggplot2)

library(pals)

# Multiple plot function
#
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  require(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                    ncol = cols, nrow = ceiling(numPlots/cols))
    print(c("nrows",ceiling(numPlots/cols)))
  }

 if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    row = 1
    col = 1
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

            print(plots[[i]], vp = viewport(layout.pos.row = row,
                                      layout.pos.col = col))

      print( c( i, row,col  ) )
      col=col+1
      if(col>cols){
        col=1
        row=row+1
      }

    }
  }
}


getval<-function(val){
  return(as.numeric(as.character(val)))
}

getrow<-function(xtab){
  return(c(xtab[1,7],xtab[2,6],xtab[3,5],xtab[4,4],xtab[5,3],xtab[6,2],xtab[7,1]))
}



# ads = xtabs(kappa_size1~plastratio+plastincr,aggregate(kappa_size1~plastratio+plastincr,data,mean))


getrow<-function(xtab){
  return(c(xtab[1,7],xtab[2,6],xtab[3,5],xtab[4,4],xtab[5,3],xtab[6,2],xtab[7,1]))
}

plot_aggdata=function(aggdata,lowlim,mid,highlim,xlab,ylab,with_legend,title,jet_color,log_scale){
jet.colors = colorRampPalette(c("#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow", "#FF7F00", "red", "#7F0000"))

if(jet_color){
#my_scale = scale_fill_gradientn(colours = parula(25),limits=c(lowlim,highlim) ,guide=guide_colourbar(barwidth=2,barheight=10))
my_scale = scale_fill_gradientn(colours = inferno(25),limits=c(lowlim,highlim) ,guide=guide_colourbar(barwidth=2,barheight=10))
# my_scale = scale_fill_gradientn(colours = jet.colors(25),limits=c(lowlim,highlim) ,guide=guide_colourbar(barwidth=2,barheight=10))

    if(log_scale){
      my_scale =   scale_fill_gradientn(limits=c(0.008,2),colours=inferno(1000),trans="log",
      breaks = c (0.01,0.1,1),guide=guide_colourbar(barwidth=2,barheight=10) )    
    }


}else{
    my_scale =   scale_fill_gradientn(colours=colorRampPalette (c ("darkblue", "white", "darkred")) (3),
    values = c (0,(mid-lowlim)/(highlim-lowlim),1),limits=c(lowlim,highlim),guide=guide_colourbar(barwidth=2,barheight=10) )

    if(log_scale){
      my_scale =   scale_fill_gradientn(limits=c(0.003,120),colours=colorRampPalette (c ("darkblue", "white", "darkred")) (1000),trans="log",
      breaks = c (0.01,1,100),guide=guide_colourbar(barwidth=2,barheight=10) )    
    }
}

# indexes_true = which(aggdata$x>0.75)
# grid_border_values=rep(0,length(aggdata$x))
# grid_border_values[indexes_true]=1
 
q <- ggplot(aggdata, aes(as.numeric(as.character(Group.1)), as.numeric(as.character(Group.2)), fill = x)) +
 geom_tile(colour=aggdata$boundary_color,size=0.8) +
  theme_bw(10) + 
  xlab("Inhibitory Connectivity (%)") + ylab("Excitatory Connectivity (%)") +
  my_scale + #xlim(0.025,1.125)+ylim(0.025,1.125) + coord_cartesian(xlim=c(0.125,1.0),ylim=c(0.125,1.0)) +
  ggtitle(title) + 
     theme(text=element_text(size=21),plot.title = element_text(lineheight=.8, face="bold",size=21))
  if(!with_legend){
    q <- q+ theme(legend.position="none",axis.text.x = element_text(angle = 90, hjust = 1),panel.margin = unit(5, "lines"))
  }else{
    q <- q+ theme(axis.text.x = element_text(angle = 90, hjust = 1),plot.margin=unit(c(2,2,2,2), "lines"))
  }
  return(q)
}

get_param_plots<-function(data,reference,inivec,minval,medval,maxval,title,jet_color=FALSE,with_legend=TRUE,boundary_type="none",log_scale=FALSE){
  aggdata <-aggregate(reference, by=list(data$incon,data$excon), 
      FUN=mean, na.rm=TRUE)

  boundary_color = rep(0.6,length(aggdata$x))

  aggdata <-aggregate(inivec, by=list(data$incon,data$excon), 
      FUN=mean, na.rm=TRUE)
  aggdata$boundary_color = boundary_color
  print(title)
  q1<-plot_aggdata(aggdata,minval,medval,maxval,"incon","excon",with_legend,title,jet_color,log_scale)
  return(q1)
}

data = read.table("results.csv",sep=';',header=TRUE)

data$excon = as.factor(data$Excitatory.Connectivity)
data$incon = as.factor(data$Inhibitory.Connectivity)

# png("DFA.png",width=500,height=400)

#png("results_paramspace.png",width=2400,height=1400)
# postscript(file="plot.eps", onefile=TRUE, horizontal=FALSE,height=6,width=21,paper="special")
pdf(file="plot.pdf", onefile=TRUE, height=6,width=21,paper="special")

p1<-get_param_plots(data,data$DFA,data$KappaSize,0.8,1,1.6,'Kappa (Avalanche sizes)')
p2<-get_param_plots(data,data$DFA,data$DFA,0.48,0.75,1.0,'DFA',jet_color=TRUE)
#p3<-get_param_plots(data,data$DFA,data$mean_act / 2500 * 1000,0,1,50,'Mean firing rate (Hz)',log_scale=TRUE,jet_color=TRUE)
p3<-get_param_plots(data,data$DFA,data$X8_12_absolute,0,0.1,1.7,'Absolute power',log_scale=TRUE,jet_color=TRUE)

# p1<-ggplot(data, aes(Multiplier.STP.time.constants, DFA)) + geom_point(colour = "black", size = 4)+labs(title = "DFA")
# p2<-ggplot(data, aes(Multiplier.STP.time.constants, KappaSize)) + geom_point(colour = "black", size = 4)+labs(title = "KappaSize")
# p3<-ggplot(data, aes(Multiplier.STP.time.constants, 1-KStat)) + geom_point(colour = "black", size = 4)+labs(title = "KStat")
# p4<-ggplot(data, aes(Multiplier.STP.time.constants, exp1)) + geom_point(colour = "black", size = 4)+labs(title = "exp1")
# p5<-ggplot(data, aes(Multiplier.STP.time.constants, exp2)) + geom_point(colour = "black", size = 4)+labs(title = "exp2")
# p6<-ggplot(data, aes(Multiplier.STP.time.constants, exp_full)) + geom_point(colour = "black", size = 4)+labs(title = "exp_full")
multiplot(p1,p2,p3,cols=3) 
dev.off()

# dev.off()

# data$kappa_dur[which(is.na(data$kappa_dur))] = 2
# data$kappa_size[which(is.na(data$kappa_size))] = 2
